<?php
/**
 * Created by JetBrains PhpStorm.
 * User: Nate
 * Date: 12/5/13
 * Time: 7:55 PM
 * Handles the 'priority' field on a database record, when performing up/down/delete/add operations on the table records
 */

class PositionSorting extends CComponent
{

    public $model;
    public $field = 'position';
    public $primary_key = 'id';
    public $condition = ''; //allow us to do things like AND user_id = 2 on the $model

    public function init() {}

    protected function getPriorityFromId($id)
    {
        $model = $this->model;
        $record = $model::model()->findByPk($id);
        return $record->{$this->field};
    }

    protected function getHighestPriority()
    {
        $model = $this->model;
        $criteria = new CDbCriteria();
        $criteria->order = $this->field . ' DESC';
        $criteria->condition = $this->condition;
        $highest_record = $model::model()->find($criteria);
        return $highest_record->{$this->field};
    }

    protected function getLowestPriority()
    {
        $model = $this->model;
        $criteria = new CDbCriteria();
        $criteria->order = $this->field . ' ASC';
        $criteria->condition = $this->condition;
        $lowest_record = $model::model()->find($criteria);
        return $lowest_record->{$this->field};
    }

    protected function updatePriority($id, $priority)
    {
        $model = $this->model;
        if($priority >= $this->getHighestPriority() + 1 && $this->hasMultipleWithPriority($this->getHighestPriority()) == false)
            return false;
        if($priority <= $this->getLowestPriority() - 1 && $this->hasMultipleWithPriority($this->getLowestPriority()) == false)
            return false;

        $record = $model::model()->findByPk($id);
        if($record)
        {
            $record->{$this->field} = $priority;
            $record->save(false);
        }
        return true;
    }

    protected function hasMultipleWithPriority($priority)
    {
        $model = $this->model;
        $criteria = new CDbCriteria();
        $criteria->condition = "{$this->field} = $priority";
        $criteria->addCondition($this->condition);
        $counted = $model::model()->count($criteria);
        if($counted > 1)
        {
            return true;
        }
        else
        {
            return false;
        }
    }

    protected function getIdFromPriority($priority)
    {
        $model = $this->model;
        $criteria = new CDbCriteria();
        $criteria->condition = "{$this->field} = $priority";
        $criteria->addCondition($this->condition);
        $other_record = $model::model()->find($criteria);
        if($other_record)
        {
            return $other_record->{$this->primary_key};
        }
        else
        {
            return false;
        }
    }

    public function move($id, $position)
    {
        $model = $this->model;
        $record_priority = $this->getPriorityFromId($id);
        //increase the priority
        if($position == 'up')
        {
            $new_priority = $record_priority - 1;
        }
        elseif($position == 'down')
        {
            $new_priority = $record_priority + 1;
        }
        //get record with same priority as this
        $other_record_id = $this->getIdFromPriority($new_priority);
        $success = $this->updatePriority($id, $new_priority);

        //decrease the priority of a field with the same priority
        if($success != false)
        {
            if($position == 'up')
            {
                $other_record_priority = $new_priority + 1;
            }
            elseif($position == 'down')
            {
                $other_record_priority = $new_priority - 1;
            }
            $this->updatePriority($other_record_id, $other_record_priority);
        }
    }

    public function add($id)
    {
        $model = $this->model;
        $this->updatePriority($id, $this->getHighestPriority() + 1);
    }

    public function delete($id)
    {
        //do some magical reordering here
    }

}
